@@ -19,6 +19,7 @@
 #include <QDBusMessage>
 #include <QDBusPendingCall>
 #include <QDesktopServices>
+#include <QFile>
 #include <QGuiApplication>
 #include <QPointer>
 #include <QQmlApplicationEngine>
@@ -33,7 +34,6 @@
 // KDE includes
 #include <KAboutData>
 #include <KAuthorized>
-#include <KConcatenateRowsProxyModel>
 #include <KConfigGroup>
 #include <KCrash>
 #include <KLocalizedContext>
@@ -100,7 +100,6 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
 
     qmlRegisterType<UnityLauncher>("org.kde.discover.app", 1, 0, "UnityLauncher");
     qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
-    qmlRegisterType<KConcatenateRowsProxyModel>("org.kde.discover.app", 1, 0, "KConcatenateRowsProxyModel");
     qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
     qmlRegisterType<PowerManagementInterface>("org.kde.discover.app", 1, 0, "PowerManagementInterface");
     qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
@@ -165,7 +164,7 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
 
 DiscoverObject::~DiscoverObject()
 {
-    delete m_engine;
+    m_engine->deleteLater();
 }
 
 bool DiscoverObject::isRoot()
