@@ -9,28 +9,27 @@
 
 #include "discovercommon_export.h"
 #include <QList>
-
+#include <QStringList>
 class QCommandLineParser;
-class QStringList;
 class AbstractResourcesBackend;
 
 class DISCOVERCOMMON_EXPORT DiscoverBackendsFactory
 {
 public:
     DiscoverBackendsFactory();
-    
-    QVector<AbstractResourcesBackend*> backend(const QString& name) const;
-    QVector<AbstractResourcesBackend*> allBackends() const;
+
+    QVector<AbstractResourcesBackend *> backend(const QString &name) const;
+    QVector<AbstractResourcesBackend *> allBackends() const;
     QStringList allBackendNames(bool whitelist = true, bool allowDummy = false) const;
     int backendsCount() const;
-    
-    static void setupCommandLine(QCommandLineParser* parser);
-    static void processCommandLine(QCommandLineParser* parser, bool test);
+
+    static void setupCommandLine(QCommandLineParser *parser);
+    static void processCommandLine(QCommandLineParser *parser, bool test);
     static void setRequestedBackends(const QStringList &backends);
     static bool hasRequestedBackends();
 
 private:
-    QVector<AbstractResourcesBackend*> backendForFile(const QString& path, const QString& name) const;
+    QVector<AbstractResourcesBackend *> backendForFile(const QString &path, const QString &name) const;
 };
 
 #endif // MUONBACKENDSFACTORY_H
