@@ -47,6 +47,7 @@
 #include <QRegularExpression>
 #include <glib.h>
 
+#include <optional>
 #include <sys/stat.h>
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
@@ -140,6 +141,24 @@ public:
         return m_remote;
     }
 
+    QList<AppStream::Component> componentsByName(const QString &name)
+    {
+        QList<AppStream::Component> comps;
+        auto findComponents = [&](const QString &name) {
+            comps = m_pool->componentsById(name);
+            if (!comps.isEmpty())
+                return;
+
+            comps = m_pool->componentsByProvided(AppStream::Provided::KindId, name);
+        };
+
+        findComponents(name);
+        if (comps.isEmpty()) {
+            findComponents(name + QLatin1String(".desktop"));
+        }
+        return comps;
+    }
+
     AppStream::Pool *m_pool = nullptr;
     QHash<FlatpakResource::Id, FlatpakResource *> m_resources;
 
@@ -150,6 +169,25 @@ private:
     const QString m_appstreamIconsDir;
 };
 
+static void populateRemote(FlatpakRemote *remote, const QString &name, const QString &url, const QString &gpgKey)
+{
+    flatpak_remote_set_url(remote, url.toUtf8().constData());
+    flatpak_remote_set_noenumerate(remote, false);
+    flatpak_remote_set_title(remote, name.toUtf8().constData());
+
+    if (!gpgKey.isEmpty()) {
+        gsize dataLen = 0;
+        g_autofree guchar *data = nullptr;
+        g_autoptr(GBytes) bytes = nullptr;
+        data = g_base64_decode(gpgKey.toUtf8().constData(), &dataLen);
+        bytes = g_bytes_new(data, dataLen);
+        flatpak_remote_set_gpg_verify(remote, true);
+        flatpak_remote_set_gpg_key(remote, bytes);
+    } else {
+        flatpak_remote_set_gpg_verify(remote, false);
+    }
+}
+
 QDebug operator<<(QDebug debug, const FlatpakResource::Id &id)
 {
     QDebugStateSaver saver(debug);
@@ -182,13 +220,47 @@ static FlatpakResource::Id idForInstalledRef(FlatpakInstalledRef *ref, const QSt
     return {appId, branch, arch};
 }
 
+static std::optional<AppStream::Metadata> metadataFromBytes(GBytes *appstreamGz, GCancellable *cancellable)
+{
+    g_autoptr(GError) localError = nullptr;
+    g_autoptr(GZlibDecompressor) decompressor = nullptr;
+    g_autoptr(GInputStream) streamGz = nullptr;
+    g_autoptr(GInputStream) streamData = nullptr;
+    g_autoptr(GBytes) appstream = nullptr;
+
+    /* decompress data */
+    decompressor = g_zlib_decompressor_new(G_ZLIB_COMPRESSOR_FORMAT_GZIP);
+    streamGz = g_memory_input_stream_new_from_bytes(appstreamGz);
+    if (!streamGz) {
+        return {};
+    }
+
+    streamData = g_converter_input_stream_new(streamGz, G_CONVERTER(decompressor));
+
+    appstream = g_input_stream_read_bytes(streamData, 0x100000, cancellable, &localError);
+    if (!appstream) {
+        qWarning() << "Failed to extract appstream metadata from bundle:" << localError->message;
+        return {};
+    }
+
+    gsize len = 0;
+    gconstpointer data = g_bytes_get_data(appstream, &len);
+
+    AppStream::Metadata metadata;
+    metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+    AppStream::Metadata::MetadataError error = metadata.parse(QString::fromUtf8((char *)data, len), AppStream::Metadata::FormatKindXml);
+    if (error != AppStream::Metadata::MetadataErrorNoError) {
+        qWarning() << "Failed to parse appstream metadata: " << error;
+        return {};
+    }
+    return metadata;
+}
+
 FlatpakBackend::FlatpakBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(AppStreamIntegration::global()->reviews())
-    , m_refreshAppstreamMetadataJobs(0)
     , m_cancellable(g_cancellable_new())
-    , m_threadPool(new QThreadPool(this))
 {
     g_autoptr(GError) error = nullptr;
 
@@ -198,9 +270,9 @@ FlatpakBackend::FlatpakBackend(QObject *parent)
     if (!setupFlatpakInstallations(&error)) {
         qWarning() << "Failed to setup flatpak installations:" << error->message;
     } else {
+        m_sources = new FlatpakSourcesBackend(m_installations, this);
         loadAppsFromAppstreamData();
 
-        m_sources = new FlatpakSourcesBackend(m_installations, this);
         SourcesModel::global()->addSourcesBackend(m_sources);
     }
 
@@ -259,7 +331,7 @@ public:
         }
 
         QNetworkRequest req(m_url);
-        req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+        req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
         auto replyGet = get(req);
         connect(replyGet, &QNetworkReply::finished, this, [this, replyGet] {
             QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(replyGet);
@@ -371,11 +443,7 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
     const QString pathApps = pathExports + QLatin1String("share/applications/");
     AppStream::Component cid;
     if (source && source->m_pool) {
-        QList<AppStream::Component> comps = source->m_pool->componentsById(name);
-        if (comps.isEmpty()) {
-            comps = source->m_pool->componentsById(name + ".desktop");
-        }
-
+        QList<AppStream::Component> comps = source->componentsByName(name);
         if (comps.isEmpty()) {
             const QString bundleId = refToBundleId(FLATPAK_REF(ref));
             comps = kFilter<QList<AppStream::Component>>(comps, [&bundleId](const AppStream::Component &comp) -> bool {
@@ -383,6 +451,22 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
             });
         }
 
+        if (comps.isEmpty()) {
+            g_autoptr(GBytes) metadata = flatpak_installed_ref_load_appdata(ref, 0, 0);
+            if (metadata) {
+                auto meta = metadataFromBytes(metadata, m_cancellable);
+                const auto componentsProvided = meta->components();
+                if (!componentsProvided.isEmpty() && name != componentsProvided.constFirst().id()) {
+                    qDebug() << "mismatch between flatpak and appstream" << name << componentsProvided.constFirst().id();
+                    comps = source->m_pool->componentsById(componentsProvided.constFirst().id());
+                }
+
+                if (comps.isEmpty()) {
+                    comps = componentsProvided;
+                }
+            }
+        }
+
         if (comps.count() >= 1) {
             Q_ASSERT(comps.count() == 1);
             cid = comps.constFirst();
@@ -432,6 +516,7 @@ QSharedPointer<FlatpakSource> FlatpakBackend::findSource(FlatpakInstallation *in
         }
     }
 
+    qWarning() << "Could not find source:" << installation << origin;
     Q_UNREACHABLE();
 }
 
@@ -455,6 +540,19 @@ FlatpakResource *FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) con
         }
     }
 
+    for (auto installation : m_installations) {
+        auto instref = flatpak_installation_get_installed_ref(installation,
+                                                              FLATPAK_REF_KIND_RUNTIME,
+                                                              runtimeInfo.at(0).toUtf8().constData(),
+                                                              runtimeInfo.at(1).toUtf8().constData(),
+                                                              runtimeInfo.at(2).toUtf8().constData(),
+                                                              m_cancellable,
+                                                              nullptr);
+        if (instref) {
+            return getAppForInstalledRef(installation, instref);
+        }
+    }
+
     // TODO if runtime wasn't found, create a new one from available info
     if (!runtime) {
         qWarning() << "could not find runtime" << runtimeName << resource;
@@ -488,38 +586,12 @@ void FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url, ResultsStream *str
 
     appstreamGz = flatpak_bundle_ref_get_appstream(bundleRef);
     if (appstreamGz) {
-        g_autoptr(GZlibDecompressor) decompressor = nullptr;
-        g_autoptr(GInputStream) streamGz = nullptr;
-        g_autoptr(GInputStream) streamData = nullptr;
-        g_autoptr(GBytes) appstream = nullptr;
-
-        /* decompress data */
-        decompressor = g_zlib_decompressor_new(G_ZLIB_COMPRESSOR_FORMAT_GZIP);
-        streamGz = g_memory_input_stream_new_from_bytes(appstreamGz);
-        if (!streamGz) {
+        const auto metadata = metadataFromBytes(appstreamGz, m_cancellable);
+        if (!metadata.has_value()) {
             return;
         }
 
-        streamData = g_converter_input_stream_new(streamGz, G_CONVERTER(decompressor));
-
-        appstream = g_input_stream_read_bytes(streamData, 0x100000, m_cancellable, &localError);
-        if (!appstream) {
-            qWarning() << "Failed to extract appstream metadata from bundle:" << localError->message;
-            return;
-        }
-
-        gsize len = 0;
-        gconstpointer data = g_bytes_get_data(appstream, &len);
-
-        AppStream::Metadata metadata;
-        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
-        AppStream::Metadata::MetadataError error = metadata.parse(QString::fromUtf8((char *)data, len), AppStream::Metadata::FormatKindXml);
-        if (error != AppStream::Metadata::MetadataErrorNoError) {
-            qWarning() << "Failed to parse appstream metadata: " << error;
-            return;
-        }
-
-        const QList<AppStream::Component> components = metadata.components();
+        const QList<AppStream::Component> components = metadata->components();
         if (components.size()) {
             asComponent = components.first();
         } else {
@@ -610,7 +682,7 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
     // There we will fetch the appstream metadata and then delete that temporary installation.
 
     g_autoptr(GError) localError = nullptr;
-    const QString path = QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1String("/discover-flatpak-temporary") + remoteName;
+    const QString path = QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1String("/discover-flatpak-temporary-") + remoteName;
     qDebug() << "Creating temporary installation" << path;
     g_autoptr(GFile) file = g_file_new_for_path(QFile::encodeName(path).constData());
     g_autoptr(FlatpakInstallation) tempInstallation = flatpak_installation_new_for_path(file, true, cancellable, &localError);
@@ -622,10 +694,10 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
     });
 
     g_autoptr(FlatpakRemote) tempRemote = flatpak_remote_new(remoteName.toUtf8());
-    FlatpakSourcesBackend::populateRemote(tempRemote,
-                                          remoteName,
-                                          settings.value(QStringLiteral("Flatpak Ref/Url")).toString(),
-                                          settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString().toUtf8());
+    populateRemote(tempRemote,
+                   remoteName,
+                   settings.value(QStringLiteral("Flatpak Ref/Url")).toString(),
+                   settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString().toUtf8());
     if (!flatpak_installation_modify_remote(tempInstallation, tempRemote, cancellable, &localError)) {
         qDebug() << "error adding temporary remote" << localError->message;
         return {asComponent};
@@ -694,15 +766,12 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
         remote = nullptr;
     }
     if (remote) {
-        m_refreshAppstreamMetadataJobs++;
+        Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(remote));
+        m_refreshAppstreamMetadataJobs.insert(remote);
         auto source = integrateRemote(preferredInstallation(), remote);
         if (source) {
             auto searchComponent = [this, stream, source, name] {
-                auto comps = source->m_pool->componentsById(name);
-                if (comps.isEmpty()) {
-                    const QString nameWithDesktop = name + QLatin1String(".desktop");
-                    comps = source->m_pool->componentsById(nameWithDesktop);
-                }
+                auto comps = source->componentsByName(name);
                 auto resources = kTransform<QVector<AbstractResource *>>(comps, [this, source](const auto &comp) {
                     return resourceForComponent(comp, source);
                 });
@@ -739,6 +808,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
 
     QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
     auto refSource = QSharedPointer<FlatpakSource>::create(this, preferredInstallation());
+    resource->setTemporarySource(refSource);
     m_flatpakSources += refSource;
     if (!runtimeUrl.isEmpty()) {
         // We need to fetch metadata to find information about required runtime
@@ -857,8 +927,10 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 {
     Q_ASSERT(flatpakInstallation);
 
-    g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, nullptr);
+    GError *error = nullptr;
+    g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, &error);
     if (!remotes) {
+        qWarning() << "failed to list remotes" << error->message;
         return false;
     }
 
@@ -873,14 +945,17 @@ void FlatpakBackend::loadRemote(FlatpakInstallation *installation, FlatpakRemote
 {
     g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, flatpak_get_default_arch());
 
+    Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(remote));
+    m_refreshAppstreamMetadataJobs.insert(remote);
+
     g_autofree char *path_str = g_file_get_path(fileTimestamp);
     QFileInfo fileInfo(QFile::encodeName(path_str));
     // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
     if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
         checkForUpdates(installation, remote);
     } else {
-        m_refreshAppstreamMetadataJobs++;
-        integrateRemote(installation, remote);
+        auto source = integrateRemote(installation, remote);
+        Q_ASSERT(findSource(installation, flatpak_remote_get_name(remote)) == source);
     }
 }
 
@@ -898,10 +973,11 @@ void FlatpakBackend::unloadRemote(FlatpakInstallation *installation, FlatpakRemo
     acquireFetching(false);
 }
 
-void FlatpakBackend::metadataRefreshed()
+void FlatpakBackend::metadataRefreshed(FlatpakRemote *remote)
 {
-    m_refreshAppstreamMetadataJobs--;
-    if (m_refreshAppstreamMetadataJobs == 0) {
+    const bool removed = m_refreshAppstreamMetadataJobs.remove(remote);
+    Q_ASSERT(removed);
+    if (m_refreshAppstreamMetadataJobs.isEmpty()) {
         for (auto installation : qAsConst(m_installations)) {
             // Load local updates, comparing current and latest commit
             loadLocalUpdates(installation);
@@ -912,39 +988,21 @@ void FlatpakBackend::metadataRefreshed()
     }
 }
 
-QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
+void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
 {
-    Q_ASSERT(m_refreshAppstreamMetadataJobs != 0);
-    for (auto source : qAsConst(m_flatpakSources)) {
-        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation) {
-            metadataRefreshed();
-            return source;
-        }
-    }
-    for (auto source : qAsConst(m_flatpakLoadingSources)) {
-        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation) {
-            metadataRefreshed();
-            return source;
-        }
-    }
-
-    auto source = QSharedPointer<FlatpakSource>::create(this, flatpakInstallation, remote);
-    if (!source->isEnabled() || flatpak_remote_get_noenumerate(remote)) {
-        m_flatpakSources += source;
-        metadataRefreshed();
-        return {};
+    if (source->m_pool) {
+        return;
     }
 
     const QString appstreamDirPath = source->appstreamDir();
     if (!QFile::exists(appstreamDirPath)) {
         qWarning() << "No" << appstreamDirPath << "appstream metadata found for" << source->name();
-        metadataRefreshed();
-        return {};
+        metadataRefreshed(source->remote());
+        return;
     }
 
     AppStream::Pool *pool = new AppStream::Pool(this);
     auto fw = new QFutureWatcher<bool>(this);
-    const auto sourceName = source->name();
     connect(fw, &QFutureWatcher<bool>::finished, this, [this, fw, pool, source]() {
         source->m_pool = pool;
         m_flatpakLoadingSources.removeAll(source);
@@ -953,7 +1011,7 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
         } else {
             qWarning() << "Could not open the AppStream metadata pool" << pool->lastError();
         }
-        metadataRefreshed();
+        metadataRefreshed(source->remote());
         acquireFetching(false);
         fw->deleteLater();
     });
@@ -968,12 +1026,41 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
     pool->setFlags(AppStream::Pool::FlagReadCollection);
     pool->setCacheFlags(AppStream::Pool::CacheFlagUseUser);
 
-    const QString subdir = flatpak_installation_get_id(flatpakInstallation) + QLatin1Char('/') + sourceName;
+    const QString subdir = flatpak_installation_get_id(source->installation()) + QLatin1Char('/') + source->name();
     pool->setCacheLocation(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/flatpak-appstream/" + subdir);
     QDir().mkpath(pool->cacheLocation());
 #endif
 
     fw->setFuture(QtConcurrent::run(&m_threadPool, pool, &AppStream::Pool::load));
+}
+
+QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
+{
+    Q_ASSERT(m_refreshAppstreamMetadataJobs.contains(remote));
+    m_sources->addRemote(remote, flatpakInstallation);
+    for (auto source : qAsConst(m_flatpakSources)) {
+        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation
+            && source->name() == flatpak_remote_get_name(remote)) {
+            createPool(source);
+            return source;
+        }
+    }
+    for (auto source : qAsConst(m_flatpakLoadingSources)) {
+        if (source->url() == flatpak_remote_get_url(remote) && source->installation() == flatpakInstallation
+            && source->name() == flatpak_remote_get_name(remote)) {
+            createPool(source);
+            return source;
+        }
+    }
+
+    auto source = QSharedPointer<FlatpakSource>::create(this, flatpakInstallation, remote);
+    if (!source->isEnabled() || flatpak_remote_get_noenumerate(remote)) {
+        m_flatpakSources += source;
+        metadataRefreshed(remote);
+        return source;
+    }
+
+    createPool(source);
     m_flatpakLoadingSources << source;
     return source;
 }
@@ -981,9 +1068,9 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
 void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 {
     g_autoptr(GError) localError = nullptr;
-    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(flatpakInstallation, m_cancellable, &localError);
+    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs_for_update(flatpakInstallation, m_cancellable, &localError);
     if (!refs) {
-        qWarning() << "Failed to get list of installed refs for listing updates:" << localError->message;
+        qWarning() << "Failed to get list of installed refs for listing local updates:" << localError->message;
         return;
     }
 
@@ -1279,7 +1366,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
     else if (filter.state == AbstractResource::Upgradeable) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-upgradeable"));
         auto f = [this, stream] {
-            auto fw = new QFutureWatcher<QMap<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>(this);
+            auto fw = new QFutureWatcher<QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>>>(this);
             connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, fw, stream]() {
                 if (g_cancellable_is_cancelled(m_cancellable)) {
                     stream->finish();
@@ -1317,7 +1404,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
             QVector<FlatpakInstallation *> installations = m_installations;
             auto cancellable = m_cancellable;
             fw->setFuture(QtConcurrent::run(&m_threadPool, [installations, cancellable] {
-                QMap<FlatpakInstallation *, QVector<FlatpakInstalledRef *>> ret;
+                QHash<FlatpakInstallation *, QVector<FlatpakInstalledRef *>> ret;
 
                 for (auto installation : installations) {
                     g_autoptr(GError) localError = nullptr;
@@ -1461,10 +1548,9 @@ bool FlatpakBackend::isTracked(FlatpakResource *resource) const
 QVector<AbstractResource *> FlatpakBackend::resourcesByAppstreamName(const QString &name) const
 {
     QVector<AbstractResource *> resources;
-    const QString nameWithDesktop = name + QLatin1String(".desktop");
     for (const auto &source : m_flatpakSources) {
         if (source->m_pool) {
-            const auto comps = source->m_pool->componentsById(name) + source->m_pool->componentsById(nameWithDesktop);
+            auto comps = source->componentsByName(name);
             resources << kTransform<QVector<AbstractResource *>>(comps, [this, source](const auto &comp) {
                 return resourceForComponent(comp, source);
             });
@@ -1530,6 +1616,58 @@ AbstractReviewsBackend *FlatpakBackend::reviewsBackend() const
     return m_reviews.data();
 }
 
+void FlatpakBackend::checkRepositories(const QMap<QString, QStringList> &names)
+{
+    auto flatpakInstallationByPath = [this](const QString &path) -> FlatpakInstallation * {
+        for (auto inst : m_installations) {
+            if (FlatpakResource::installationPath(inst) == path)
+                return inst;
+        }
+        return nullptr;
+    };
+
+    g_autoptr(GError) localError = nullptr;
+    for (auto it = names.begin(), itEnd = names.end(); it != itEnd; ++it) {
+        FlatpakInstallation *installation = flatpakInstallationByPath(it.key());
+        for (const QString &name : qAsConst(*it)) {
+            auto remote = flatpak_installation_get_remote_by_name(installation, name.toUtf8(), m_cancellable, &localError);
+            if (!remote) {
+                qWarning() << "Could not find remote" << name << "in" << it.key();
+                continue;
+            }
+            loadRemote(installation, remote);
+        }
+    }
+}
+
+FlatpakRemote *FlatpakBackend::installSource(FlatpakResource *resource)
+{
+    g_autoptr(GCancellable) cancellable = g_cancellable_new();
+
+    auto remote = flatpak_installation_get_remote_by_name(preferredInstallation(), resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
+    if (remote) {
+        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(preferredInstallation());
+        return nullptr;
+    }
+
+    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
+    populateRemote(remote,
+                   resource->comment(),
+                   resource->getMetadata(QStringLiteral("repo-url")).toString(),
+                   resource->getMetadata(QStringLiteral("gpg-key")).toString());
+    if (!resource->branch().isEmpty()) {
+        flatpak_remote_set_default_branch(remote, resource->branch().toUtf8().constData());
+    }
+
+    g_autoptr(GError) error = nullptr;
+    if (!flatpak_installation_add_remote(preferredInstallation(), remote, false, cancellable, &error)) {
+        Q_EMIT passiveMessage(i18n("Failed to add source '%1': %2", resource->flatpakName(), error->message));
+        qWarning() << "Failed to add source " << resource->flatpakName() << error->message;
+        return nullptr;
+    }
+    return remote;
+}
+
 Transaction *FlatpakBackend::installApplication(AbstractResource *app, const AddonList &addons)
 {
     Q_UNUSED(addons);
@@ -1538,21 +1676,32 @@ Transaction *FlatpakBackend::installApplication(AbstractResource *app, const Add
 
     if (resource->resourceType() == FlatpakResource::Source) {
         // Let source backend handle this
-        FlatpakRemote *remote = m_sources->installSource(resource);
+        FlatpakRemote *remote = installSource(resource);
         if (remote) {
             resource->setState(AbstractResource::Installed);
-            // Make sure we update appstream metadata first
-            // FIXME we have to let flatpak to return the remote as the one created by FlatpakSourcesBackend will not have appstream directory
-            g_autoptr(FlatpakRemote) repo =
-                flatpak_installation_get_remote_by_name(resource->installation(), flatpak_remote_get_name(remote), m_cancellable, nullptr);
-            loadRemote(resource->installation(), repo);
+            auto name = flatpak_remote_get_name(remote);
+            g_autoptr(FlatpakRemote) remote = flatpak_installation_get_remote_by_name(resource->installation(), name, m_cancellable, nullptr);
+            loadRemote(resource->installation(), remote);
         }
         return nullptr;
     }
 
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
+    connect(transaction, &FlatpakJobTransaction::repositoriesAdded, this, &FlatpakBackend::checkRepositories);
     connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource](Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
+            if (auto tempSource = resource->temporarySource()) {
+                auto source = findSource(resource->installation(), resource->origin());
+                resource->setTemporarySource({});
+                const auto id = resource->uniqueId();
+                source->m_resources.insert(id, resource);
+
+                tempSource->m_resources.remove(id);
+                if (tempSource->m_resources.isEmpty()) {
+                    const bool removed = m_flatpakSources.removeAll(tempSource) || m_flatpakLoadingSources.removeAll(tempSource);
+                    Q_ASSERT(removed);
+                }
+            }
             updateAppState(resource);
         }
     });
@@ -1577,8 +1726,7 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
     }
 
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::RemoveRole);
-    connect(transaction, &FlatpakJobTransaction::repositoriesAdded, m_sources, &FlatpakSourcesBackend::checkRepositories);
-
+    connect(transaction, &FlatpakJobTransaction::repositoriesAdded, this, &FlatpakBackend::checkRepositories);
     connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource](Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
             updateAppSize(resource);
@@ -1590,21 +1738,30 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
 void FlatpakBackend::checkForUpdates()
 {
     for (auto source : qAsConst(m_flatpakSources)) {
-        checkForUpdates(source->installation(), source->remote());
+        if (source->remote()) {
+            Q_ASSERT(!m_refreshAppstreamMetadataJobs.contains(source->remote()));
+            m_refreshAppstreamMetadataJobs.insert(source->remote());
+            checkForUpdates(source->installation(), source->remote());
+        }
     }
 }
 
 void FlatpakBackend::checkForUpdates(FlatpakInstallation *installation, FlatpakRemote *remote)
 {
-    m_refreshAppstreamMetadataJobs++;
-    if (flatpak_remote_get_disabled(remote)) {
-        integrateRemote(installation, remote);
+    Q_ASSERT(remote);
+    const bool needsIntegration = m_refreshAppstreamMetadataJobs.contains(remote);
+    if (flatpak_remote_get_disabled(remote) || flatpak_remote_get_noenumerate(remote)) {
+        if (needsIntegration) {
+            integrateRemote(installation, remote);
+        }
         return;
     }
 
     FlatpakRefreshAppstreamMetadataJob *job = new FlatpakRefreshAppstreamMetadataJob(installation, remote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
-    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [this] {
+    if (needsIntegration) {
+        connect(job, &FlatpakRefreshAppstreamMetadataJob::jobRefreshAppstreamMetadataFinished, this, &FlatpakBackend::integrateRemote);
+    }
+    connect(job, &FlatpakRefreshAppstreamMetadataJob::finished, this, [=] {
         acquireFetching(false);
     });
 
@@ -1623,7 +1780,7 @@ HelpfulError *FlatpakBackend::explainDysfunction() const
         return new HelpfulError(QStringLiteral("emblem-error"), i18n("There are no Flatpak sources."), m_sources->actions());
     }
     for (const auto &source : m_flatpakSources) {
-        if (!source->m_pool->lastError().isEmpty()) {
+        if (source->m_pool && !source->m_pool->lastError().isEmpty()) {
             return new HelpfulError(QStringLiteral("emblem-error"), i18n("Failed to load \"%1\" source", source->name()));
         }
     }
